package org.rcisoft.business.bbanner.dao;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.rcisoft.business.bbanner.dto.AddOrUpdateDTO;
import org.rcisoft.business.bbanner.dto.FindBannerPaginDTO;
import org.rcisoft.business.bbanner.dto.TopDownDTO;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.springframework.boot.Banner;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author
 * @date 2019/9/10
 */
@Repository
public interface BBannerRepository extends BaseMapper<BBanner> {


    /**
     * 根据条件查找
     * @param param
     * @return
     */
    @Select("<script>select bb.* , bl.lesson_name lessonName,bl.default_url lessonDefaultUrl from b_banner bb " +
            " left join b_lesson bl on bb.course_id = bl.business_id " +
            " where bb.del_flag='0' and bb.flag = '1' " +
            " <if test=\"businessId !=null and businessId != ''\">and bb.business_id = #{businessId}</if>" +
            " <if test=\"title !=null and title != ''\">and bb.title like CONCAT('%',#{title},'%')</if>" +
            " <if test=\"isExternalLink !=null and isExternalLink != ''\">and bb.is_external_link=#{isExternalLink}</if>" +
            " <if test=\"lineType !=null and lineType != ''\">and bb.line_type=#{lineType}</if>" +

            " and bb.corp_id = #{corpId} "+

            " order by CAST(bb.banner_sort AS UNSIGNED) asc " +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BBanner> queryBanners(FindBannerPaginDTO param);

    @Select("<script>select bb.* , bl.lesson_name lessonName ,bl.default_url from b_banner bb " +
            " left join b_lesson bl on bb.course_id = bl.business_id " +

//            " and bl.corp_id = #{corpId} "+

            " and bl.del_flag='0' and bl.flag = '1'" +
            " where bb.del_flag='0' and bb.flag = '1' " +
            " and bb.business_id = #{businessId}" +

//            " and bb.corp_id = #{corpId}  "+

            "</script>")
    @ResultMap(value = "BaseResultMap" )
    BBanner queryOne(String businessId);

    /**
     * 按id查询个数
     * @param id
     * @return
     */
    @Select("<script>select count(business_id) from b_banner where del_flag='0' and flag = '1' and business_id=#{id} " +
            "</script>")
    int selectCountById(String id);

    /**
     * 查询Banner个数
     * @return
     */
    @Select("<script>select count(1) from b_banner where " +
            " del_flag = 0 and flag = 1 " +
            " and corp_id = #{corpId} " +
            " and is_push != 1" +
            "</script>")
    int selectCountBanner(AddOrUpdateDTO param);

    /**
     * 获取sort最大值 +1
     * @return
     */
    @Select("<script>select DISTINCT IFNULL((select max(banner_sort) from b_banner where " +
            "del_flag = 0 and flag = 1 ),0)+1 as banner_sort</script>")
    int selectMaxSort();

    /**
     * 更新
     * @param model
     * @return
     */
    @Update({"<script>",
            "update b_banner",
            "  <set>",
            "    <if test='updateBy != null'>update_by=#{updateBy},</if>",
            "    <if test='updateDate != null'>update_date=#{updateDate},</if>",
            "    <if test='delFlag != null'>del_flag=#{delFlag},</if>",
            "    <if test='flag != null'>flag=#{flag},</if>",
            "    <if test='remarks != null'>remarks=#{remarks},</if>",
            "    <if test='title != null'>title=#{title},</if>",
            "    <if test='imageUrl != null'>image_url=#{imageUrl},</if>",
            "    <if test='isExternalLink != null'>is_external_link=#{isExternalLink},</if>",
            "    <if test='bannerSort != null'>banner_sort=#{bannerSort},</if>",
            "   external_url=#{externalUrl},",
            "   line_type=#{lineType},",
            "   course_id=#{courseId} ",
            "  </set>",
            " where business_id=#{businessId}",
            "</script>"})
    int updateById(BBanner model);


    /**
     * 逻辑删除
     * @param id
     * @return
     */
    @Update("update b_banner set del_flag = 1 where business_id = #{id}")
    int deleteBanner(String id);


    @Select("<script>select * from b_banner where banner_sort &gt; #{bannerSort} " +
            " and del_flag = 0 and flag = 1 " +
            "ORDER BY banner_sort asc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap" )
    BBanner getBigChapter(TopDownDTO param);

    @Update("<script>update b_banner set banner_sort = #{bannerSort} where business_id = #{businessId}</script>")
    int updateSortById(@Param("businessId") String businessId, @Param("bannerSort") int bannerSort);

    @Select("<script>select * from b_banner where banner_sort &lt; #{bannerSort} " +
            "and del_flag = 0 and flag = 1 " +
            "ORDER BY banner_sort desc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap" )
    BBanner getlessChapter(TopDownDTO param);


    //重新排序
    @Update("<script> <foreach collection=\"bannerOrderMap.entrySet()\" index=\"key\" item=\"value\" separator=\";\"> " +
            "update b_banner " +
            "set banner_sort = #{value} " +
            " WHERE business_id = #{key} " +
            "</foreach></script>")
    int updateBannerOrderByid(@Param("bannerOrderMap") Map<String,Object> bannerOrderMap);
}
