package org.rcisoft.business.bchapter.controller;

import cn.hutool.core.bean.BeanUtil;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.IsCompleteEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bfile.dto.uploadDTO;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.enums.RoleFlagEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//import org.rcisoft.business.bsl.service.BSlService;

/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Api(tags = "02 章节")
@RestController
@RequestMapping("/BChapter")
@Slf4j
public class BChapterController extends PaginationController<BChapter> {
    @Autowired
    private BChapterService bChapterService;

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Autowired
    private Global global;

    @ApiOperation(value = "201 添加/编辑章节", notes = "businessId为空时是添加方法，不为空时是修改方法")
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid ChapterDTO chapterDTO, BindingResult bindingResult, uploadDTO dto) {
        PersistModel data = bChapterService.persist(chapterDTO,curUser.getCorpId(),dto);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                chapterDTO);
    }


    @ApiOperation(value = "202 删除章节", notes = "根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "章节Id", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser,String id) {
        PersistModel data = bChapterService.removeBChapter(id,curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value = "203 分页查询章节", notes = "根据条件分页查询章节", response = QueryChapterListResDTO.class)
    @PostMapping(value = "/queryBChaptersByPagination")
    public Result queryBChaptersByPagination(CurUser curUser,@Valid QueryChapterListDTO param,BindingResult br) {
        bChapterService.queryBChaptersByPagination(getPaginationUtility(), param,curUser.getCorpId());
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "204 查询章节", notes = "根据lessonId查询章节", response = QueryChapterListResDTO.class)
    @GetMapping(value = "/queryBChaptersWithoutAuth")
    public Result queryBChapters(CurUser curUser,@Valid QueryChapterListDTO param,BindingResult br) {
        Map<String, Object> map = new HashedMap();
        map.put("chapterList", bChapterService.queryBChapters(param,curUser.getCorpId()));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }
    @ApiOperation(value = "205 查询章节带进度", notes = "根据lessonId查询章节带进度", response = QueryChapterListResDTO.class)
    @GetMapping(value = "/queryBChaptersWithProgress")
    public Result queryBChaptersWithProgress(CurUser curUser, @RequestParam String lessonId, BindingResult br, String isExternalShare,String chapterId) {
        Map<String, Object> map = new HashedMap();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapterService.queryBChaptersWithProgress(lessonId,curUser,isExternalShare,chapterId));
    }


    @ApiOperation(value = "206 将ppt转为pdf", notes = "流 输出ppt转为pdf")
    @GetMapping("/getFile")
    public Result getFile(CurUser curUser, @Valid String chapterId, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.changeFileToPdf(chapterId);
        //读取路径下面的文件
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }


    @ApiOperation(value = "207 预览文件", notes = "(预览时传)")
    @GetMapping("/getFileAtView")
    public Result getFileAtView(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
        FileToPdfAtViewDTO dto = bChapterService.changeFileToPdfAtView(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                dto);
    }

    @ApiOperation(value = "208 下载文件 返回服务器地址", notes = "下载文件 返回服务器地址")
    @PostMapping("/getDownLoadUrl")
    public Result getDownLoadUrl(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.getDownLoadUrl(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }


    @ApiOperation(value = "209 判断文件是否可读", notes = "判断文件是否可读(培训 需要预览的附件)")
    @GetMapping("/fileReadable")
    public Result fileReadable(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
        String res = bChapterService.fileReadable(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                res);
    }

    @ApiOperation(value = "210 判断文件类型与后缀名是否相符(根据http路径)", notes = "判断文件类型与后缀名是否相符")
    @GetMapping("/checkFileHeader")
    public Result checkFileHeader(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
//        String res = bChapterService.checkFileHeader(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                file);
    }
    @ApiOperation(value = "211 判断文件类型与后缀名是否相符（根据文件）", notes = "判断文件类型与后缀名是否相符")
    @PostMapping("/checkHeaderFromFile")
    public Result checkHeaderFromFile(CurUser curUser, MultipartFile file){
        return Result.builder(new PersistModel(bChapterService.checkHeaderFromFile(file)),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                file.getOriginalFilename());
    }

    @ApiOperation(value = "212 批量下载文件 返回服务器Zip包地址", notes = "下载文件 批量返回服务器Zip包地址")
    @ApiImplicitParam(name = "businessId", value = "文件json", required = true, dataType = "varchar")
    @PostMapping("/getBatchDownload")
    public Result getBatchDownload(CurUser curUser, @Valid String fileJson, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.getBatchDownload(fileJson);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }


    @ApiOperation(value = "213 根据传入的课程ids 批量下载文件 返回服务器Zip包地址", notes = "下载文件 批量返回服务器Zip包地址")
    @ApiImplicitParam(name = "ids", value = "课程ids(用英文逗号隔开)", required = true, dataType = "varchar")
    @PostMapping("/getBatchsDownload")
    public Result getBatchsDownload(CurUser curUser,@RequestParam String ids, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.getBatchsDownload(ids);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }

    @ApiOperation(value = "214 根据传入的课程id 单个课程下载文件 返回服务器Zip包地址", notes = "下载文件 批量返回服务器Zip包地址")
    @ApiImplicitParam(name = "id", value = "课程id", required = true, dataType = "varchar")
    @PostMapping("/getLessonDownload")
    public Result getLessonDownload(CurUser curUser,@RequestParam String id, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.getLessonDownload(id);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }

//    @ApiOperation(value = "209 查询章节带进度(外部分享)", notes = "查询章节带进度(外部分享)", response = QueryChapterListResDTO.class)
//    @GetMapping(value = "/queryShareBChaptersWithProgress")
//    public Result queryShareBChaptersWithProgress(CurUser curUser, @RequestParam String lessonId, BindingResult br) {
//        Map<String, Object> map = new HashedMap();
//        return Result.builder(new PersistModel(1),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapterService.queryBChaptersWithProgress(lessonId,curUser));
//    }


//
//    @ApiOperation(value = "更改章节状态", notes = "根据章节ID更改章节状态")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar")})
//    @PostMapping(value = "/updateChapterState")
//    public Result updateChapterState(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateChapterState(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }

//    @ApiOperation(value = "章节向上调顺序", notes = "章节向上调顺序")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
//            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
//    @PostMapping(value = "/updateTopChapter")
//    public Result updateTopChapter(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateTopChapter(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }
//
//    @ApiOperation(value = "章节向下调顺序", notes = "章节向下调顺序")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
//            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
//    @PostMapping(value = "/updateDownChapter")
//    public Result updateDownChapter(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateDownChapter(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }

}
