package org.rcisoft.business.bcourse.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.bcourse.dto.QueryCourseResDTO;
import org.rcisoft.business.bcourse.entity.BCourse;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


@Repository
public interface BCourseRepository extends BaseMapper<BCourse> {


    @Select("<script>SELECT * FROM b_course where del_flag != 1 and flag = 1 and course_level = 1 " +

            " and corp_id = #{corpId} " +

            "<if test=\"cName!=null and cName != ''\"> and c_name like CONCAT('%',#{cName},'%') </if>" +
            "ORDER BY sort</script>")
    @ResultMap(value = "ChildListResultMap")
    List<QueryCourseResDTO> queryCourses(Map<String, Object> param);

    @Select("select *from b_course where del_flag != 1 and pid = #{pid} ORDER BY sort ")
    @ResultMap(value = "ChildListResultMap")
    List<QueryCourseResDTO> queryCoursesByPid(String parentId);

    @Select("SELECT * from b_course WHERE del_flag != 1 AND flag = 1 " +

            " and corp_id = #{corpId} " +

            " AND c_name=#{cName} ")
    List<BCourse> selectOneByName(@Param("cName") String id, @Param("corpId") String corpId);

//    @Update("UPDATE b_course SET del_flag='1' WHERE business_id = #{businessId}")
    @Update("<script>" +
            "<foreach collection=\"courseIds\" item=\"item\" separator=\";\">" +
            "UPDATE b_course SET del_flag = '1' WHERE business_id = #{item}" +
            "</foreach>" +
            "</script>")
    int removeCourse(@Param("courseIds") List<String> courseIds);

    @Update("UPDATE b_course SET c_name = #{cName} WHERE business_id = #{businessId}")
    int updateCourse(BCourse bCourse);

    @Select("select DISTINCT course_level from b_course where business_id = #{pid} " +

            "and corp_id = #{corpId} "
    )
    int queryParentLevel(@Param("pid") String pid, @Param("corpId") String corpId);

    @Select("select * from b_course where course_level = 1 and del_flag != '1' and flag ='1' " +

            " and corp_id = #{corpId} " +

            " ORDER BY sort limit #{num}")
    @ResultMap(value = "ChildListResultMap")
    List<QueryCourseResDTO> queryFirstLevel(@Param("corpId") String corpId,@Param("num") int num);

    @Select("select DISTINCT IFNULL((" +
            "select max(sort) from b_course where pid = #{pid} " +

            " and corp_id = #{corpId} " +

            "and  del_flag = 0 and flag = 1),0)+1 as sort")
    int getSort(@Param("pid") String pid,@Param("corpId") String corpId);

    /**
     * 查询所有分类存到List中
     *
     * @return
     */
    @Select("select * from b_course where del_flag != '1' " +

            "and corp_id = #{corpId} " +

            "and flag ='1' ")
    @ResultMap(value = "AllCourseResultMap")
    List<AllCourseDTO> findAllCourse(@Param("corpId") String corpId);

    @Update("<script>" +
            "<foreach collection=\"map\" item=\"item\" separator=\";\">" +
            "update b_course " +
            "set " +
            "pid = #{item.pid}, " +
            "sort = #{item.sort}, " +
            "course_level = #{item.courseLevel} " +
            "where business_id = #{item.businessId} " +
            "</foreach>" +
            "</script>")
    int sortCourser(@Param("map") List<BCourse> map);
}