package org.rcisoft.business.bcourse.service.impl;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONArray;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.rcisoft.business.bcourse.dao.BCourseRepository;
import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.bcourse.dto.QueryCourseResDTO;
import org.rcisoft.business.bcourse.entity.BCourse;
import org.rcisoft.business.bcourse.service.BCourseService;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.service.impl.BLessonServiceImpl;
import org.rcisoft.business.blesson.util.Recursion;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BCourseServiceImpl implements BCourseService {

    @Autowired
    private BCourseRepository bCourseRepository;

    @Autowired
    private BLessonServiceImpl bLessonService;


    @Override
    public List<Map<String,Object>> queryBCourse(BCourse model) {
        Map param = new HashMap<String, Object>();
        if (model.getCName() != null && !"".equals(model.getCName())) {
            param.put("cName", model.getCName());
        }
        param.put("corpId", model.getCorpId());
        List<QueryCourseResDTO> ret = bCourseRepository.queryCourses(param);
        ret = addNewFolder(ret);
        List<Map<String,Object>> maplist = new ArrayList<>();
        for(QueryCourseResDTO dto :ret){
            Map<String,Object> map = new HashMap<>();
            if(dto.getChildren().size()!=0){
                List<Map<String,Object>> maplist2 = new ArrayList<>();
                for(QueryCourseResDTO dto2:dto.getChildren()){
                    Map<String,Object> map2 = new HashMap<>();
                    if(dto2.getChildren().size()!=0){
                        List<Map<String,Object>> maplist3 = new ArrayList<>();
                        for(QueryCourseResDTO dto3:dto2.getChildren()){
                            Map<String,Object> map3 = new HashMap<>();
                            map3.put("pid",dto3.getPid());
                            map3.put("key",dto3.getKey());
                            map3.put("courseLevel",dto3.getCourseLevel());
                            map3.put("sort",dto3.getSort());
                            map3.put("title",dto3.getTitle());
                            map3.put("newCount",dto3.getNewCount());
                            maplist3.add(map3);
                        }
                        map2.put("children",maplist3);
                    }
                    map2.put("pid",dto2.getPid());
                    map2.put("key",dto2.getKey());
                    map2.put("courseLevel",dto2.getCourseLevel());
                    map2.put("sort",dto2.getSort());
                    map2.put("title",dto2.getTitle());
                    map2.put("newCount",dto2.getNewCount());
                    maplist2.add(map2);
                }
                map.put("children",maplist2);
            }
            map.put("pid",dto.getPid());
            map.put("key",dto.getKey());
            map.put("courseLevel",dto.getCourseLevel());
            map.put("sort",dto.getSort());
            map.put("title",dto.getTitle());
            map.put("newCount",dto.getNewCount());
            maplist.add(map);
        }
        return maplist;
    }

    @Override
    public List<QueryCourseResDTO> queryBCourseByPagination(PageUtil pageUtil, BCourse model) {
        Map param = new HashMap<String, Object>();
        if (model.getCName() != null && !"".equals(model.getCName())) {
            param.put("cName", model.getCName());
        }
        param.put("corpId", model.getCorpId());
        List<QueryCourseResDTO> resDTOS = bCourseRepository.queryCourses(param);
        return addNewFolder(resDTOS);
    }

    @Override
    public PersistModel insertCourse(BCourse bCourse) {

        if (bCourse.getCName() == null || bCourse.getCName().equals("")) {
            throw new ServiceException("课程分类名不能为空");
        }
//        List<BCourse> list = bCourseRepository.selectOneByName(bCourse.getCName(), bCourse.getCorpId());
//        if (list.size() > 0) {
//            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
//        }

        if (bCourse.getPid() == null || "".equals(bCourse.getPid())) {
            bCourse.setPid("00000000");
            bCourse.setCourseLevel("1");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if ("00000000".equals(bCourse.getPid())) {
            bCourse.setCourseLevel("1");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if (bCourseRepository.queryParentLevel(bCourse.getPid(), bCourse.getCorpId()) == 1) {//通过getpid 获取上一级的level
            bCourse.setCourseLevel("2");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if (bCourseRepository.queryParentLevel(bCourse.getPid(), bCourse.getCorpId()) == 2) {
            bCourse.setCourseLevel("3");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else {
            throw new ServiceException(ResultServiceEnums.COURSE_ERROR);
        }
        UserUtil.setCurrentPersistOperation(bCourse);
        return new PersistModel(bCourseRepository.insert(bCourse), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel removeCourse(String corpId, String id) {

        List<AllCourseDTO> allCourse = null;

        if (StringUtils.isNotEmpty(id)) {
            allCourse = this.findAllCourse(corpId);
        }
        List<String> courseIds = null;
        if (allCourse != null) {
            //查询出所有子分类
            courseIds = Recursion.FindSons(id, allCourse);
            //将一级分类放入分类集合中
            courseIds.add(id);
        }
        return new PersistModel(
                bCourseRepository.removeCourse(courseIds), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel updateCourse(BCourse bCourse) {
        if (bCourse.getCName() == null || bCourse.getCName().equals("")) {
            throw new ServiceException("课程分类名不能为空");
        }
//        List<BCourse> list = bCourseRepository.selectOneByName(bCourse.getCName(), bCourse.getCorpId());
//        if (list.size() > 0) {
//            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
//        }
        UserUtil.setCurrentMergeOperation(bCourse);
        return new PersistModel(bCourseRepository.updateCourse(bCourse), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel sortCourse(String json) {
//        Map<String,Object> maps = (Map) JSON.parse(json);
//        List<BCourse> bCourses = new ArrayList<>();
//        for (String obj : maps.keySet()) {
//            BCourse bCourse = new BCourse();
//            String value = maps.get(obj).toString();
//            bCourse.setBusinessId(obj.toString());
//            String[] split = StringUtils.split(value, ",");
//            bCourse.setPid(split[0]);
//            bCourse.setSort(Integer.parseInt(split[1]));
//            bCourse.setCourseLevel(split[2]);
//            bCourses.add(bCourse);
//        }
//        int line = bCourseRepository.sortCourser(bCourses);
        List<Map<String, String>> listObjectFir = (List<Map<String, String>>) JSONArray.parse(json);
        List<BCourse> bCourses = new ArrayList<>();
        for (Map<String, String> mapList : listObjectFir) {
            BCourse bCourse = new BCourse();

            for (Map.Entry<String, String> str : mapList.entrySet()) {
                if (str.getKey().equals("key")) {
                    bCourse.setBusinessId(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("pid")) {
                    bCourse.setPid(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("sort")) {
                    bCourse.setSort(Integer.parseInt(String.valueOf(str.getValue())));
                }
                if (str.getKey().equals("courseLevel")) {
                    bCourse.setCourseLevel(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("children")) {

                    String child = String.valueOf(str.getValue());
                    List<Map<String, String>> list1 = (List<Map<String, String>>) JSONArray.parse(child);
                    for (Map<String, String> mapList1 : list1) {

                        BCourse bCourse1 = new BCourse();
                        for (Map.Entry<String, String> str1 : mapList1.entrySet()) {
                            if (str1.getKey().equals("key")) {
                                bCourse1.setBusinessId(str1.getValue());
                            }
                            if (str1.getKey().equals("pid")) {
                                bCourse1.setPid(str1.getValue());
                            }
                            if (str1.getKey().equals("sort")) {
                                bCourse1.setSort(Integer.parseInt(String.valueOf(str1.getValue())));
                            }
                            if (str1.getKey().equals("courseLevel")) {
                                bCourse1.setCourseLevel(String.valueOf(str1.getValue()));
                            }
                            if (str1.getKey().equals("children")) {

                                String child2 = String.valueOf(str1.getValue());
                                List<Map<String, String>> list2 = (List<Map<String, String>>) JSONArray.parse(child2);
                                for (Map<String, String> mapList2 : list2) {
                                    BCourse bCourse2 = new BCourse();
                                    for (Map.Entry<String, String> str2 : mapList2.entrySet()) {
                                        if (str2.getKey().equals("key")) {
                                            bCourse2.setBusinessId(str2.getValue());
                                        }
                                        if (str2.getKey().equals("pid")) {
                                            bCourse2.setPid(str2.getValue());
                                        }
                                        if (str2.getKey().equals("sort")) {
                                            bCourse2.setSort(Integer.parseInt(String.valueOf(str2.getValue())));
                                        }
                                        if (str2.getKey().equals("courseLevel")) {
                                            bCourse2.setCourseLevel(String.valueOf(str2.getValue()));
                                        }
                                    }
                                    bCourses.add(bCourse2);
                                }
                            }
                        }
                        bCourses.add(bCourse1);
                    }
                }
            }
            bCourses.add(bCourse);
        }
        int line = bCourseRepository.sortCourser(bCourses);
        return new PersistModel(line);
    }

    @Override
    public List<AllCourseDTO> findAllCourse(String corpId) {
        return bCourseRepository.findAllCourse(corpId);
    }

    @Override
    public List<QueryCourseResDTO> queryFirstLevel(String corpId,int num) {
        List<QueryCourseResDTO> dtoList = bCourseRepository.queryFirstLevel(corpId,num);
        return addNewFolder(dtoList);
    }

    /**
     * 生产分类中 new 的个数
     * @param resDTOS  一级分类带子分类集合
     * @return
     */
    public List<QueryCourseResDTO> addNewFolder(List<QueryCourseResDTO> resDTOS){
        log.info("-----------addNewFolder----------当前接口查询的一级分类个数---------"+resDTOS.size()+"--------------------");
        List<BLesson> lessons = new ArrayList<>();
        List<BLesson> oldLessons = bLessonService.queryHomeBLessons(UserUtil.getCurUser());//查询首页显示的所有 课程（isNew 已赋值）
        //筛选有分类的 new 课程
        for(BLesson lesson : oldLessons){
            if("1".equals(lesson.getIsNew()) && (lesson.getCourseId() != null || !"".equals(lesson.getCourseId())))
                lessons.add(lesson);
        }
        for (QueryCourseResDTO dto : resDTOS){  //一级分类
            int num = 0; //属于当前分类的课程数
            int secNum = 0 ;//下层分类中的课程数
            for (BLesson b : lessons){
                if(b.getCourseId().equals(dto.getKey()))
                    num++;
            }
            secNum+=num;//添加分类直属 new课程的数量
            if (dto.getChildren() != null){
                for (QueryCourseResDTO dto1 : dto.getChildren()){ //二级分类
                    int num1 = 0; //二级分类下直属的课程
                    int secNum1 = 0 ;//二级分类下 三级分类中的的 new课程数
                    for (BLesson b : lessons){
                        if(b.getCourseId().equals(dto1.getKey()))
                            num1++;
                    }
                    secNum1 += num1; //将二级分类中的 直属课 添加到一级分类中
                    if (dto.getChildren() != null){
                        for (QueryCourseResDTO dto2 : dto1.getChildren()){  //三级分类
                            int num2 = 0;
                            for (BLesson b : lessons){
                                if(b.getCourseId().equals(dto2.getKey()))
                                    num2++;
                            }
                            log.info("-----------addNewFolder----------三级分类--"+dto2+"--的new个数-------"+num2+"--------------------");
                            dto2.setNewCount(num2);//将三级分类包含的 new课数量添加到 分类中
                            secNum1+=num2; //将三级分类中的 new课数量添加到 二级分类中
                        }
                    }
                    log.info("-----------addNewFolder---------二级分类--"+dto1+"--的new个数-------"+secNum1+"--------------------");
                    dto1.setNewCount(secNum1);
                    secNum += secNum1;//将二级分类下三级分类中的数量添加到 二级分类中
                }
            }
            log.info("-----------addNewFolder----------一级分类--"+dto+"--的new个数-------"+secNum+"--------------------");
            dto.setNewCount(secNum);
        }

        return resDTOS;
    }

}
