package org.rcisoft.business.bexternallink.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.rcisoft.business.bexternallink.dto.AddOrUpdateDTO;
import org.rcisoft.business.bexternallink.dto.FindListDTO;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bexternallink.dao.BExternalLinkRepository;
import org.rcisoft.business.bexternallink.entity.BExternalLink;
import org.rcisoft.business.bexternallink.service.BExternalLinkService;


import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.io.InputStream;
import java.net.URL;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-13 13:28:38.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BExternalLinkServiceImpl implements BExternalLinkService {

    @Autowired
    private BExternalLinkRepository bExternalLinkRepository;


    /**
    * 保存 bExternalLink
    * @param addOrUpdateDTO
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(AddOrUpdateDTO addOrUpdateDTO){

        BExternalLink bExternalLink = new BExternalLink();
        BeanUtils.copyProperties(addOrUpdateDTO, bExternalLink);

        int line = 0;
        //外部链接但外部URL为空或长度为0 抛异常
        if (StringUtils.isEmpty(bExternalLink.getExternalUrl()))
            throw new ServiceException(ResultServiceEnums.JUMP_ADDRESS_NULL);
        else if (StringUtils.isNotBlank(bExternalLink.getExternalUrl())){
//            try {
//                URL url = new URL(bExternalLink.getExternalUrl());
//                InputStream in = ((URL) url).openStream();
//            } catch (Exception e1) {
//                throw new ServiceException(ResultServiceEnums.ILLEGAL_URL);
//            }
        }
        //TODO
        if (bExternalLink.getBusinessId() != null) {
            //按ID查询外链个数
            if (bExternalLinkRepository.selectCountById(bExternalLink.getBusinessId()) == 0){
                throw new ServiceException(ResultServiceEnums.NO_DATA);
            }
            UserUtil.setCurrentMergeOperation(bExternalLink);
            line = bExternalLinkRepository.updateByPrimaryKeySelective(bExternalLink);
        } else {
            UserUtil.setCurrentPersistOperation(bExternalLink);
            //查询外链是否大于等于10个，大于等于十个不允许添加
            int countBanner =  bExternalLinkRepository.selectCountExternalLink(bExternalLink.getCorpId());
            if(countBanner >= 15){
                throw new ServiceException(ResultServiceEnums.NOT_MORE_COUNT);
            }
            //查询最大排序值+1后放入外链中
            int sort = bExternalLinkRepository.selectMaxSort();
            bExternalLink.setSort(sort);
            line = bExternalLinkRepository.insertSelective(bExternalLink);
        }
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param id
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(String id){
        BExternalLink bExternalLink = new BExternalLink(id);
        UserUtil.setCurrentMergeOperation(bExternalLink);
        bExternalLink.setDeleted();
        int line = bExternalLinkRepository.logicalDelete(bExternalLink);
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bExternalLink
    * @param id
    * @return
    */
    public BExternalLink findById(String id){
        return bExternalLinkRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bExternalLink
    * @param findListDTO
    * @return
    */
    public List<BExternalLink> findAllByPagination(PageUtil<BExternalLink> paginationUtility,
                                                   FindListDTO findListDTO){
        return bExternalLinkRepository.queryBExternalLinks(findListDTO);
    }

    @Override
    public int updateExternalLinkOrder(String bannerOrderMap) {
        //获取将字符串中的主键和位置对应关系转成Map
        Map parse = (Map) JSON.parse(bannerOrderMap);
        int i = bExternalLinkRepository.updateExternalLinkOrder(parse);
        return i;
    }


}
