package org.rcisoft.business.bmessage.service.impl;

import cn.hutool.core.date.DateUtil;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dao.BPChapterRepository;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bfile.dao.BFileRepository;
import org.rcisoft.business.bfile.dao.BPFileRepository;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dao.BPLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.enums.ReleaseStateEnum;
import org.rcisoft.business.blessonlog.dao.BLessonLogRepository;
import org.rcisoft.business.blessonlog.entity.BLessonLog;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bmessage.service.BMessageService;


import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-25 10:55:03.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BMessageServiceImpl implements BMessageService {

    @Autowired
    private BMessageRepository bMessageRepository;
    @Autowired
    private BLessonLogRepository bLessonLogRepository;

    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BChapterRepository bChapterRepository;
    @Autowired
    private BFileRepository bFileRepository;

    @Autowired
    private BPLessonRepository bpLessonRepository;
    @Autowired
    private BPChapterRepository bpChapterRepository;
    @Autowired
    private BPFileRepository bpFileRepository;


    /**
    * 保存 bMessage
    * @param bMessage
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BMessage bMessage){
        //增加操作
        UserUtil.setCurrentPersistOperation(bMessage);
        int line = bMessageRepository.insertSelective(bMessage);
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bMessage
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BMessage bMessage){
        UserUtil.setCurrentMergeOperation(bMessage);
        bMessage.setDeleted();
        int line = bMessageRepository.logicalDelete(bMessage);
        return new PersistModel(line);
    }

    /**
    * 修改 bMessage
    * @param bMessage
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(CurUser curUser , BMessage bMessage){

        UserUtil.setCurrentMergeOperation(bMessage);
        int line = bMessageRepository.updateByPrimaryKeySelective(bMessage);
        return new PersistModel(line);
    }

    /**
    * 修改 bMessage
    * @param bMessage
    * @return
    */
//    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel updateSycn(CurUser curUser , BMessage bMessage){
        //先查询 原数据
        BMessage oldBMessage = bMessageRepository.selectByPrimaryKey(bMessage.getBusinessId());
        Calendar calendar = DateUtil.calendar(oldBMessage.getUpdateDate()); //获取上次操作时间
        calendar.add(Calendar.MINUTE,1); //将此文件和当前时间对比
        Calendar now = Calendar.getInstance(); //获取当前时间
        Long sec = (now.getTimeInMillis() - calendar.getTimeInMillis())/1000;
        if(sec <= 0L)
            throw new ServiceException(165,"请"+Math.abs(sec)+"秒后再次操作！");
        //保存课程日志
        sendLog(curUser,bMessage);
        //开启共享时同步课程
        if("1".equals(bMessage.getTimeValue()))
            sycnBPLesson(curUser);
        UserUtil.setCurrentMergeOperation(bMessage);
        int line = bMessageRepository.updateByPrimaryKeySelective(bMessage);
        return new PersistModel(line);
    }


    /**
    * 根据id查询 bMessage
    * @param id
    * @return
    */
    public BMessage findById(String id){
        return bMessageRepository.selectByPrimaryKey(id);
    }

    @Override
    public List<BMessage> findAll(CurUser curUser) {
        return bMessageRepository.queryBMessages(curUser);
    }

//    /**
//    * 分页查询 bMessage
//    * @param bMessage
//    * @return
//    */
//    public List<BMessage> findAllByPagination(PageUtil<BMessage> paginationUtility,
//        BMessage bMessage){
//        bMessage.setStart();
//        bMessage.setNotDeleted();
//        return bMessageRepository.queryBMessages(bMessage);
//    }

    /**
     * 生成课程日志
     * @param curUser
     * @param bMessage
     */
    public int sendLog(CurUser curUser ,BMessage bMessage){
        //查询当前企业所有已发布已关闭和删除（曾经是已关闭和已发布）的课程
        List<BLesson> lessons = bLessonRepository.getLessons(curUser.getCorpId());
        List<BLessonLog> list = new ArrayList<>();
        //将操作日志添加到每一个课程
        for(BLesson lesson : lessons){
            BLessonLog log = BLessonLog.generate("1".equals(bMessage.getTimeValue())?"开启资源共享":"关闭资源共享",lesson.getBusinessId());
            list.add(log);
        }
        int line = bLessonLogRepository.insertList(list);
        return line;
    }

    /**
     * 将企业的发布课程保存到 平台中
     * @param curUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public void sycnBPLesson(CurUser curUser){
        //step1 查询当前企业已发布的课程
        List<BLesson> lessons = bLessonRepository.getLessons(curUser.getCorpId());
        List<BChapter> chapters = new ArrayList<>();
        List<BFile> files  = new ArrayList<>();
        //step2 根据课程查询对应的章节和课件
        lessons.forEach(lesson->{
            if(DelStatus.DELETED.getStatus().equals(lesson.getDelFlag())){
                lesson.setDelFlag(DelStatus.NORMAL.getStatus());
                lesson.setReleaseState(ReleaseStateEnum.DELETE.getCode());
            }
            List<BChapter> chapterList = bChapterRepository.queryBChaptersByLessonId(lesson.getBusinessId());
            List<BFile> fileList =  bFileRepository.getFileByLessonId(lesson.getBusinessId());
            chapters.addAll(chapterList);
            files.addAll(fileList);
        });
        //step3 将数据插入 缓存课程、章节、课件 表中(全删全加)
        bpLessonRepository.deleteList(lessons);
        bpChapterRepository.deleteList(chapters);
        bpFileRepository.deleteList(files);
        bpLessonRepository.insertList(lessons);
        bpChapterRepository.insertList(chapters);
        bpFileRepository.insertList(files);

    }

}
