package org.rcisoft.business.breleasevalue.controller;


/*固定导入*/
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.bbanner.dto.FindBannerPaginDTO;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.rcisoft.business.breleasevalue.dto.*;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.breleasevalue.entity.BReleaseValue;
import org.rcisoft.business.breleasevalue.service.BReleaseValueService;

import java.util.List;

/**
* Created by  on 2019-11-13 13:28:39.
*/
@Api(tags = "15 发布积分")
@RestController
@RequestMapping("breleasevalue")
@Slf4j
public class BReleaseValueController extends PaginationController<BReleaseValue> {

    @Autowired
    private BReleaseValueService bReleaseValueServiceImpl;

    @ApiOperation(value = "1501 分页查询发布积分设置", notes = "根据条件分页查询", response = BReleaseValue.class)
    @GetMapping(value = "/queryByPagination")
    public Result queryByPagination(CurUser curUser,@Valid ReleaseValueResDto dto ,BindingResult br) {
        bReleaseValueServiceImpl.setReleaseValueResDtoIds(dto);
        bReleaseValueServiceImpl.queryByPagination(getPaginationUtility(), dto);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value="1502 添加、编辑积分管理", notes="添加编辑接口(businessId为空是添加)")
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid BReleaseValueDto bReleaseValueDto, BindingResult bindingResult) {
        PersistModel data = bReleaseValueServiceImpl.save(bReleaseValueDto);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
                bReleaseValueDto);
    }

    @ApiOperation(value="1503 删除积分管理", notes="删除")
    @ApiImplicitParam(name = "businessId", value = "主键id", required = true, dataType = "varchar")
    @PostMapping(value = "/delete")
    public Result delete(CurUser curUser, String businessId ) {
        PersistModel data = bReleaseValueServiceImpl.delete(businessId,curUser.getCorpId());
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="1504 分页查询课程、培训积分设置页面", notes="分页查询课程、培训积分设置页面",response = LessonValueDto.class)
    @GetMapping(value = "/queryLessonValue")
    public Result queryLessonValue(CurUser curUser, @Valid LessonValueSettingQueryDto dto ,BindingResult bindingResult) {
        bReleaseValueServiceImpl.queryLessonValueByPagination(getPaginationUtility(),curUser,dto);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value="1505 编辑课程、培训积分", notes="编辑课程、培训积分")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "主键id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "valueConsume", value = "课程、培训 消耗积分", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "valueGain", value = "课程、培训 获得积分", required = true, dataType = "varchar")})
    @PostMapping(value = "/updateLessonValue")
    public Result updateLessonValue(CurUser curUser, String businessId,String valueConsume,String  valueGain) {
        PersistModel data = bReleaseValueServiceImpl.updateLessonValue(businessId,valueConsume,valueGain);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="1506 分页查询 用户积分统计页面", notes="分页查询 用户积分设置页面" ,response = UserValueDto.class)
    @GetMapping(value = "/queryUserValue")
    public Result queryUserValue(CurUser curUser, UserValueResDto dto) {
        bReleaseValueServiceImpl.queryUserValueByPagination(getPaginationUtility(),dto);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value="1507 查询用户发布课程、培训积分", notes="查询用户发布课程、培训积分" )
    @GetMapping(value = "/queryPersonReleaseValue")
    public Result queryPersonReleaseValue(CurUser curUser) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bReleaseValueServiceImpl.queryPersonReleaseValue(curUser));
    }

}
