package org.rcisoft.business.brstudentchapter.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dto.QueryChapterListDTO;
import org.rcisoft.business.bchapter.dto.QueryChapterListResDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.blesson.dao.BLessonPersonRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dto.LessonAndUserDTO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.enums.IsFinishEnum;
import org.rcisoft.business.blesson.service.impl.BLessonServiceImpl;
import org.rcisoft.business.blesson.util.LessonSyncUtil;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.business.brstudentchapter.dto.BRStudentChapterDto;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.brstudentchapter.dao.BRStudentChapterRepository;
import org.rcisoft.business.brstudentchapter.entity.BRStudentChapter;
import org.rcisoft.business.brstudentchapter.service.BRStudentChapterService;


import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.NumberFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Semaphore;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-9-25 10:01:43.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BRStudentChapterServiceImpl implements BRStudentChapterService {

    @Autowired
    private BRStudentChapterRepository bRStudentChapterRepository;
    @Autowired
    private BLessonPersonRepository bLessonPersonRepository;
    @Autowired
    private BChapterRepository bChapterRepository;
    @Autowired
    private BChapterService bChapterServiceImpl;
    @Autowired
    BRStudentChapterRepository brStudentChapterRepository;
    @Autowired
    BLessonServiceImpl bLessonServiceImpl;
    @Autowired
    BLessonRepository bLessonRepository;

    /**
    *
    * @param brStudentChapterDto
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public Map save(BRStudentChapterDto brStudentChapterDto, CurUser curUser){
        int line = 0;
        String currentProgress = "";
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMaximumFractionDigits(0);
        synchronized (LessonSyncUtil.getSynUserDTO(brStudentChapterDto.getStudentId())) {
            // 查詢是否已經開始觀看此章節
            List<BRStudentChapter> brStudentChapterList = bRStudentChapterRepository.queryByStuIdAndChapter(brStudentChapterDto.getChapterId(),brStudentChapterDto.getStudentId());
            // 沒有開始看-新增信息
            if (null != brStudentChapterList && brStudentChapterList.size() >0 && brStudentChapterList.get(0) != null){
                double old = brStudentChapterList.get(0).getProgress() ;
                BRStudentChapter brStudentChapter = this.dealBrstudentChapter(brStudentChapterList.get(0), brStudentChapterDto);
                // 判断进度 是否大于数据库已经存在的进度
                double now = brStudentChapter.getProgress();
                if (now >= old){ // 进度大于以前的进度
                    now = now > 1 ? 1: now < 0 ? 0 : now;
                    currentProgress = (now*100)+"%";
                    if (brStudentChapterDto.getMaxPosition() != null){
                        brStudentChapter.setMaxPosition(brStudentChapterDto.getMaxPosition());
                    }
                    line = bRStudentChapterRepository.updateById(brStudentChapter);
                }else { // 反之，只需更新最近看的时间
                    brStudentChapter.setProgress(old);
                    old = old > 1 ? 1: old < 0 ? 0 : old;
                    currentProgress = (old*100)+"%";
                    line = bRStudentChapterRepository.updateById(brStudentChapter);
                }
            }else {  // 更新
                BRStudentChapter brStudentChapter = new BRStudentChapter();
                brStudentChapter.setBusinessId(IdGen.uuid());
                brStudentChapter = this.dealBrstudentChapter(brStudentChapter, brStudentChapterDto);
                brStudentChapter.setStudentId(brStudentChapterDto.getStudentId());
                brStudentChapter.setChapterId(brStudentChapterDto.getChapterId());
                // 规避 复数 和超100% 的情况
                double progress = brStudentChapter.getProgress() > 1 ? 1: brStudentChapter.getProgress() < 0 ? 0 : brStudentChapter.getProgress();
                currentProgress = (progress*100)+"%";
                line = bRStudentChapterRepository.insertSelective(brStudentChapter);
            }
        }
        // b_lesson_person
        LessonAndUserDTO addMy = new LessonAndUserDTO();
        addMy.setUserId(brStudentChapterDto.getStudentId());
        addMy.setLessonId(brStudentChapterDto.getLessonId());
        List<BLessonPerson>  bLessonPersonList = bLessonPersonRepository.selectByPersonAndLesson(addMy);
        // 将课程总进度存进数据库
        QueryChapterListDTO model = new QueryChapterListDTO();
        model.setLessonId(brStudentChapterDto.getLessonId());
        model.setStudentId(brStudentChapterDto.getStudentId());
        List<QueryChapterListResDTO> queryChapterListResDTOS = bChapterServiceImpl.queryChapterListResDTO(model, curUser.getCorpId());
        int i = 0;
        float percent = 0;
        for (QueryChapterListResDTO queryChapterListResDTO : queryChapterListResDTOS) {
            for (QueryChapterListResDTO childList : queryChapterListResDTO.getChildList()) {
                // 遍历学生章节中间表 查询学生观看章节进度
                List<BRStudentChapter> brStudentChapters = brStudentChapterRepository.queryByStuIdAndChapter(childList.getBusinessId(), curUser.getUserId());
                if (null != brStudentChapters && brStudentChapters.size() > 0 && brStudentChapters.get(0) != null) {
                    childList.setProgress(brStudentChapters.get(0).getProgress() * 100 + "%");
                    childList.setCurrentLocation(brStudentChapters.get(0).getCurrentLocation());
                    percent += brStudentChapters.get(0).getProgress();
                }
                i++;
            }
        }
        String totalProgress = "";
        if (i == 0){
            totalProgress = "0%";
        }else {
            // 规避 复数 和超100% 的情况
            float tempTotalProgress = Float.parseFloat(numberFormat.format((float) percent / (float) i * 100));
            totalProgress = tempTotalProgress > 100 ? "100%" : tempTotalProgress < 0 ? "0%" : tempTotalProgress + "%";
//            totalProgress = tempTotalProgress + "%";
        };
        BLessonPerson bLessonPerson = new BLessonPerson();
        // 判斷是否已經開始觀看此課程
        if (null != bLessonPersonList && bLessonPersonList.size() >0 && bLessonPersonList.get(0) != null){
            bLessonPerson = this.dealBLessonPerson(bLessonPersonList.get(0),brStudentChapterDto,totalProgress);
            //TODO 判断是否为已学完 曾经是否学完过 奖励积分
            BLesson bLesson = bLessonRepository.selectByPrimaryKey(brStudentChapterDto.getLessonId());
            if (bLesson != null  && IsFinishEnum.FINISHED.getCode().equals(bLessonPerson.getIsFinish()) && !("1".equals(bLessonPerson.getEverFinished()))){
                //查询课程信息
                if(StringUtils.isNotEmpty(bLesson.getValueGain()) && Long.parseLong(bLesson.getValueGain()) > 0){
                    BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学完!@#课程",bLesson.getValueGain(),"0",bLesson.getBusinessId());
                    bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,null);
                }
                bLessonPerson.setEverFinished("1");
            }
            bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }else {

            bLessonPerson.initModel();
            bLessonPerson = this.dealBLessonPerson(bLessonPerson,brStudentChapterDto,totalProgress);
            bLessonPerson.setPersonId(brStudentChapterDto.getStudentId());
            bLessonPerson.setLessonId(brStudentChapterDto.getLessonId());
            //TODO 判断是否为已学完 曾经是否学完过 奖励积分
            BLesson bLesson = bLessonRepository.selectByPrimaryKey(brStudentChapterDto.getLessonId());
            if (bLesson != null  && IsFinishEnum.FINISHED.getCode().equals(bLessonPerson.getIsFinish()) && !("1".equals(bLessonPerson.getEverFinished()))){
                //查询课程信息
                BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学完!@#课程",bLesson.getValueGain(),"0",bLesson.getBusinessId());
                bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,null);
                bLessonPerson.setEverFinished("1");
            }
             // 保存课程学习进度
            line = bLessonPersonRepository.insertSelective(bLessonPerson) ;
        }
        // 获取章节总进度 存库
        BChapter bChapter = bChapterRepository.selectByPrimaryKey(brStudentChapterDto.getChapterId());
        if (null!=bChapter && StringUtils.isNotBlank(bChapter.getBusinessId())){
            bChapter.setCourseTime(brStudentChapterDto.getDuration().toString());
            line = bChapterRepository.updateByPrimaryKeySelective(bChapter) > line ? bChapterRepository.updateByPrimaryKeySelective(bChapter): line;
        }
        Map map = new HashMap();
        // 规避 复数 和超100% 的情况
        map.put("currentProgress", currentProgress);
        return map;
    }

    // b_r_student_chapter
    private BRStudentChapter dealBrstudentChapter(BRStudentChapter brStudentChapter, BRStudentChapterDto brStudentChapterDto){
        //前端未修改前暂时跳过
        if (brStudentChapterDto.getMaxPosition() == null ){
            return brStudentChapter;
        }

        if (brStudentChapterDto.getMaxPosition().equals(brStudentChapterDto.getDuration())){
            brStudentChapter.setProgress(1);
        }else{
            brStudentChapter.setProgress((Math.floor((float)brStudentChapterDto.getMaxPosition()/brStudentChapterDto.getDuration()*100))/100);
        }
        if (!brStudentChapterDto.getMaxPosition().equals(0) && brStudentChapter.getProgress() < 0.01){
            brStudentChapter.setProgress(0.01);
        }
        brStudentChapter.setCurrentLocation(brStudentChapterDto.getCurrent());
        return  brStudentChapter;
    }


    // b_lesson_person
    private BLessonPerson dealBLessonPerson(BLessonPerson bLessonPerson, BRStudentChapterDto brStudentChapterDto, String totalProgress){
        String isFinish;
        if (totalProgress.contains("100")) {
            isFinish = "2";
        }else {
            isFinish = "1";
        }
        bLessonPerson.setChapterId(brStudentChapterDto.getChapterId());
        bLessonPerson.setLearnProgress(totalProgress);  // 保存课程学习进度
        if ("2".equals(isFinish))
            bLessonPerson.setFinishDate(new Date());
        bLessonPerson.setIsFinish(isFinish);
        return  bLessonPerson;
    }

    /**
    * 根据id查询 bRStudentChapter
    * @param id
    * @return
    */
    public BRStudentChapter findById(String id){
        return bRStudentChapterRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bRStudentChapter
    * @param bRStudentChapter
    * @return
    */
//    public List<BRStudentChapter> findAllByPagination(PageUtil<BRStudentChapter> paginationUtility,
//        BRStudentChapter bRStudentChapter){
//        return bRStudentChapterRepository.queryBRStudentChapters(bRStudentChapter);
//    }

}
