package org.rcisoft.sys.role.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.role.dto.AddOrUpdateRoleDTO;
import org.rcisoft.sys.role.dto.FindRolePaginDTO;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by lcy on 16/7/30.
 */
@Api(tags = "10 用户角色")
@RestController
@RequestMapping("/role")
public class SysRoleController extends PaginationController<SysRole> {

    @Autowired
    private SysRoleService sysRoleServiceImpl;


    @ApiOperation(value="1001 分页查询角色", notes="分页查询角色")
    @GetMapping(value = "/queryRoles")
    public GridModel queryRolesByPagination(CurUser curUser){
        sysRoleServiceImpl.queryRolesByPagination(getPaginationUtility(), curUser.getCorpId());
        GridModel gridModel = getGridModelResponse();
        return  gridModel;
    }

    @ApiOperation(value="1002 查询全部角色", notes="查询全部角色")
    @GetMapping(value = "/queryRolesAll")
    public Result queryRolesAll(CurUser curUser){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                sysRoleServiceImpl.queryRolesAll(curUser.getCorpId()));

    }

    @ApiOperation(value="1003 数据权限分配", notes="数据权限分配")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "角色id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "0：仅可看自己 1：可看全部 2：查看指定", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "departIds", value = "部门id，逗号间隔", required = true, dataType = "varchar")})
    @PostMapping(value = "/roleData")
    public Result roleData(CurUser curUser,@Valid SysRole sysRole,BindingResult br){
        return Result.builder(sysRoleServiceImpl.updateRoleData(sysRole),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                sysRole);
    }
    
}
