package org.rcisoft.sys.role.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.dto.InitAddAdminDTO;
import org.rcisoft.sys.role.dto.RoleMenuDTO;
import org.rcisoft.sys.role.dto.UserRoleDTO;
import org.rcisoft.sys.role.entity.SysRole;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created with family.
 * author: cy
 * Date: 16/6/2
 * Time: 上午10:03
 * description:
 */
@Repository
public interface SysRoleRepository extends BaseMapper<SysRole> {
    /**
     * 分页查询 roles
     *
     * @param
     * @return
     */
    @Select("<script>select * from tm_admin_role  " +
            "  where r_status = 1   " +
            "  and r_type not in (1,0)   " +
            "  and corp_id = #{corpId} " +
            "  order by r_type desc , r_createdate asc</script>")
    @ResultMap(value = "SelectAllAndUserNum")
    List<SysRole> queryRoles(@Param("corpId") String corpId);

    @Select("<script>select * from tm_admin_role  " +
            "  where r_status = 1   " +
            "  and corp_id = #{corpId} " +
            "  order by r_type desc , r_createdate asc</script>")
    @ResultMap(value = "SelectAllAndUserNum")
    List<SysRole> queryAllRoles(@Param("corpId") String corpId);

    //获得此角色的用户数
    int countUserNum(String roleId);

    @Select("<script>select * from tm_admin_role  where "
            + "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryRolesSelect(SysRole sysRole);

    /**
     * 根据角色id  删除对应权限
     *
     * @param roleId
     * @return
     */
    @Delete("<script>delete from s_role_menu where role_id = #{roleId}</script>")
    int deleteRoleMenus(@Param("roleId") String roleId);


    /**
     * 根据角色id  删除对应部门
     *
     * @param roleId
     * @return
     */
    @Delete("<script>delete from s_dept_role where role_id = #{roleId}</script>")
    int deleteRoleDepts(@Param("roleId") String roleId);

    /**
     * 批量插入         角色_权限
     *
     * @param menuRole
     * @return
     */
    int insertBatchRoleMenu(List<Map> menuRole);


    /**
     * 批量插入         角色_部门
     *
     * @param deptRole
     * @return
     */
    int insertBatchRoleDept(List<Map> deptRole);

    /**
     * 分页查询 roles
     *
     * @param sysRole
     * @return
     */
    @Select(" <script>select "
            + " dr.BUSINESS_ID drid,r.BUSINESS_ID rid,r.name rname,r.code rcode,d.BUSINESS_ID did,d.name dname,d.dept_code deptcode, "
            + " IF(( "
            + " SELECT count(*) from s_dept_role_user dru where dru.USER_ID=#{userId} "
            + " and dru.DEPT_ROLE_ID=dr.BUSINESS_ID"
            + " )>0,'1','0') as checked "
            + " FROM  s_dept_role dr "
            + " LEFT JOIN s_dept d ON d.BUSINESS_ID = dr.DEPT_ID "
            + " LEFT JOIN s_role r on dr.ROLE_ID = r.BUSINESS_ID</script>")
    @ResultMap(value = "drResultMap")
    List<DeptRole> queryUserRolesByPagination(@Param("userId") String userId, SysRole sysRole);


    /**
     * 分页查询 roles
     *
     * @param sysRole
     * @return
     */
    @Select("<script>SELECT " +
            " sysRole.BUSINESS_ID, " +
            " sysRole.NAME, " +
            " sysRole.CODE, " +
            " sysRole.DATA_SCOPE, " +
            " sysRole.FLAG, " +
            " sysRole.DEL_FLAG, " +
            " sysRole.CREATE_BY, " +
            " sysRole.create_date, " +
            "IF ( " +
            " ( " +
            "  SELECT " +
            "   count(*) " +
            "  FROM " +
            "   s_dept_ROLE dr " +
            "  WHERE " +
            "   dr.ROLE_ID = sysRole.BUSINESS_ID " +
            "  AND ur.USER_ID = #{deptId}  " +
            " ) > 0, " +
            "  '1', " +
            "  '0' " +
            " ) AS checked " +
            "FROM " +
            " s_role sysRole " +
            "WHERE " +
            " DEL_FLAG = '0' and FLAG = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryDeptRolesByPagination(@Param("deptId") String deptId, SysRole sysRole);

    @Delete("<script>DELETE from s_dept_role_user where DEPT_ROLE_ID="
            + "("
            + "SELECT BUSINESS_ID from s_dept_role  where ROLE_ID=#{roleId}"
            + ")</script>")
    int deleteDeptRoleUserByroleId(@Param("roleId") String roleId);

    /**
     * 删除岗位
     *
     * @param roleId
     * @return
     */
    @Delete("<script>DELETE from s_dept_role where ROLE_ID=#{roleId}</script>")
    int deleteDeptRoleByRoleId(@Param("roleId") String roleId);

    /**
     * 删除权限
     *
     * @param roleId
     * @return
     */
    @Delete("<script>DELETE from s_role_menu where ROLE_ID=#{roleId}</script>")
    int deleteRoleMenuByRoleId(@Param("roleId") String roleId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.business_id = #{businessId} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> findRolesByUserId(String businessId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.login_name = #{username} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryCodeByUsername(String username);


    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.business_id = #{businessId} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryCodeByBusinessId(String businessId);

    @Select("<script>select * from s_role where flag = '1' and del_flag = '0' "
            + "and (code = #{code} or role_name = #{roleName}) "
            + "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryRolesByCodeOrName(SysRole sysRole);

    @Delete("<script>DELETE from s_r_user_role where role_id=#{id}</script>")
    int deleteUserRoleByRoleId(String id);

    @Select("<script>select * from s_role where flag = '1' and del_flag = '0' "
            + "and role_name = #{roleName}"
            + "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryRolesByName(SysRole sysRole);


    @Select("SELECT count(*) FROM s_menu \n" +
            "LEFT JOIN s_role_menu  ON s_menu.BUSINESS_ID = s_role_menu.MENU_ID " +
            "LEFT JOIN s_role  ON s_role_menu.ROLE_ID = s_role.business_id " +
            "WHERE s_menu.MODEL_ID = #{modelId} AND ROLE_ID = #{roleId}")
    int queryRoleByRoleCode(@Param("modelId") String modelId, @Param("roleId") String roleId);

    @Select("<script>select count(1) " +
            "from tm_admin_role " +
            "where corp_id = #{corpId} " +
            "and r_type = 0 " +
            "</script>")
    int queryAdminInCorp(String corpId);

    @Insert("<script>insert into tm_admin_role(r_id,r_name,r_status,r_createdate,r_privilege,type,r_type,corp_id) " +
            "values(#{rId},#{rName},#{rStatus},#{rCreatedate},#{rPrivilege},#{type},#{rType},#{corpId}) " +
            "</script>")
    int addAdminRole(InitAddAdminDTO param);

    @Insert("<script>INSERT INTO s_role_menu " +
            "(business_id,role_id,menu_id)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.roleId},#{item.menuId})" +
            "</foreach></script>")
    int addRoleMenu(List<RoleMenuDTO> param);

    @Insert("<script>insert into s_r_user_role(business_id,user_id,role_id) " +
            "values" +
            "<foreach collection=\"param\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.userId},#{item.roleId})" +
            "</foreach></script>")
//            "(#{businessId},#{userId},#{roleId}) " +
//            "</script>")
    int addUserRole(@Param("param") List<UserRoleDTO> param);

    @Update("update tm_admin_role set type = #{type} , depart_ids = #{departIds} where r_id = #{businessId}")
    int updateRoleData(SysRole sysRole);

    //删除当前管理员对应的用户
    @Delete("delete from s_r_user_role where role_id = #{roleId} ")
    int deleteUserRole(String roleId);

    //获取当前企业的 管理员角色id
    @Select("select * from tm_admin_role where corp_id = #{corpId} and r_type = #{rType} ")
    String getRoleId(@Param("corpId") String corpId,@Param("rType") String rType);

    //获取 智学开通的企业 id
    @Select("select DISTINCT corp_id from tm_admin_role where r_type = 0")
    List<String> getCorpIds();
}
